@@ -9,7 +9,7 @@ import uuid
 from dataclasses import dataclass, field
 from datetime import datetime
 from types import ModuleType
-from typing import Any, Callable, Collection, Dict, List, Optional, Tuple
+from typing import Any, Callable, Collection, Dict, List, Optional, Set, Tuple, Union
 
 import pandas as pd
 
@@ -73,7 +73,29 @@ class Variable:
 
 
 class Driver(object):
-    """This class orchestrates creating and executing the DAG to create a dataframe."""
+    """This class orchestrates creating and executing the DAG to create a dataframe.
+
+    .. code-block:: python
+
+        from hamilton import driver
+        from hamilton import base
+
+        # 1. Setup config or invariant input.
+        config = {}
+
+        # 2. we need to tell hamilton where to load function definitions from
+        import my_functions
+        # or programmatically (e.g. you can script module loading)
+        module_name = 'my_functions'
+        my_functions = importlib.import_module(module_name)
+
+        # 3. Determine the return type -- default is a pandas.DataFrame.
+        adapter = base.SimplePythonDataFrameGraphAdapter() # See GraphAdapter docs for more details.
+
+        # These all feed into creating the driver & thus DAG.
+        dr = driver.Driver(config, module, adapter=adapter)
+
+    """
 
     def __init__(
         self,
@@ -93,6 +115,7 @@ class Driver(object):
         if adapter is None:
             adapter = base.SimplePythonDataFrameGraphAdapter()
         error = None
+        self.graph_modules = modules
         try:
             self.graph = graph.FunctionGraph(*modules, config=config, adapter=adapter)
             self.adapter = adapter
@@ -143,24 +166,30 @@ class Driver(object):
                 if logger.isEnabledFor(logging.DEBUG):
                     logger.debug(f"Error caught in processing telemetry: {e}")
 
-    def _node_is_required_by_anything(self, node_: node.Node) -> bool:
+    def _node_is_required_by_anything(self, node_: node.Node, node_set: Set[node.Node]) -> bool:
         """Checks dependencies on this node and determines if at least one requires it.
 
         Nodes can be optionally depended upon, i.e. the function parameter has a default value. We want to check that
         of the nodes the depend on this one, at least one of them requires it, i.e. the parameter is not optional.
 
         :param node_: node in question
+        :param node_set: checks that we traverse only nodes in the provided set.
         :return: True if it is required by any downstream node, false otherwise
         """
         required = False
         for downstream_node in node_.depended_on_by:
+            if downstream_node not in node_set:
+                continue
             _, dep_type = downstream_node.input_types[node_.name]
             if dep_type == node.DependencyType.REQUIRED:
                 return True
         return required
 
     def validate_inputs(
-        self, user_nodes: Collection[node.Node], inputs: typing.Optional[Dict[str, Any]] = None
+        self,
+        user_nodes: Collection[node.Node],
+        inputs: typing.Optional[Dict[str, Any]] = None,
+        nodes_set: Collection[node.Node] = None,
     ):
         """Validates that inputs meet our expectations. This means that:
         1. The runtime inputs don't clash with the graph's config
@@ -168,14 +197,17 @@ class Driver(object):
 
         :param user_nodes: The required nodes we need for computation.
         :param inputs: the user inputs provided.
+        :param nodes_set: the set of nodes to use for validation; Optional.
         """
         if inputs is None:
             inputs = {}
+        if nodes_set is None:
+            nodes_set = set(self.graph.nodes.values())
         (all_inputs,) = (graph.FunctionGraph.combine_config_and_inputs(self.graph.config, inputs),)
         errors = []
         for user_node in user_nodes:
             if user_node.name not in all_inputs:
-                if self._node_is_required_by_anything(user_node):
+                if self._node_is_required_by_anything(user_node, nodes_set):
                     errors.append(
                         f"Error: Required input {user_node.name} not provided "
                         f"for nodes: {[node.name for node in user_node.depended_on_by]}."
@@ -194,14 +226,14 @@ class Driver(object):
 
     def execute(
         self,
-        final_vars: List[str],
+        final_vars: List[Union[str, Callable, Variable]],
         overrides: Dict[str, Any] = None,
         display_graph: bool = False,
         inputs: Dict[str, Any] = None,
     ) -> Any:
         """Executes computation.
 
-        :param final_vars: the final list of variables we want to compute.
+        :param final_vars: the final list of outputs we want to compute.
         :param overrides: values that will override "nodes" in the DAG.
         :param display_graph: DEPRECATED. Whether we want to display the graph being computed.
         :param inputs: Runtime inputs to the DAG.
@@ -217,8 +249,9 @@ class Driver(object):
         start_time = time.time()
         run_successful = True
         error = None
+        _final_vars = self._create_final_vars(final_vars)
         try:
-            outputs = self.raw_execute(final_vars, overrides, display_graph, inputs=inputs)
+            outputs = self.raw_execute(_final_vars, overrides, display_graph, inputs=inputs)
             result = self.adapter.build_result(**outputs)
             return result
         except Exception as e:
@@ -229,9 +262,41 @@ class Driver(object):
         finally:
             duration = time.time() - start_time
             self.capture_execute_telemetry(
-                error, final_vars, inputs, overrides, run_successful, duration
+                error, _final_vars, inputs, overrides, run_successful, duration
             )
 
+    def _create_final_vars(self, final_vars: List[Union[str, Callable, Variable]]) -> List[str]:
+        """Creates the final variables list - converting functions names as required.
+
+        :param final_vars:
+        :return: list of strings in the order that final_vars was provided.
+        """
+        _final_vars = []
+        errors = []
+        module_set = {_module.__name__ for _module in self.graph_modules}
+        for final_var in final_vars:
+            if isinstance(final_var, str):
+                _final_vars.append(final_var)
+            elif isinstance(final_var, Variable):
+                _final_vars.append(final_var.name)
+            elif isinstance(final_var, Callable):
+                if final_var.__module__ in module_set:
+                    _final_vars.append(final_var.__name__)
+                else:
+                    errors.append(
+                        f"Function {final_var.__module__}.{final_var.__name__} is a function not in a "
+                        f"module given to the driver. Valid choices are {module_set}."
+                    )
+            else:
+                errors.append(
+                    f"Final var {final_var} is not a string, a function, or a driver.Variable."
+                )
+        if errors:
+            errors.sort()
+            error_str = f"{len(errors)} errors encountered:\n  " + "\n  ".join(errors)
+            raise ValueError(error_str)
+        return _final_vars
+
     def capture_execute_telemetry(
         self,
         error: Optional[str],
@@ -291,7 +356,7 @@ class Driver(object):
         """
         nodes, user_nodes = self.graph.get_upstream_nodes(final_vars, inputs)
         self.validate_inputs(
-            user_nodes, inputs
+            user_nodes, inputs, nodes
         )  # TODO -- validate within the function graph itself
         if display_graph:  # deprecated flow.
             logger.warning(
@@ -320,7 +385,7 @@ class Driver(object):
     @capture_function_usage
     def display_all_functions(
         self, output_file_path: str, render_kwargs: dict = None, graphviz_kwargs: dict = None
-    ):
+    ) -> Optional["graphviz.Digraph"]:  # noqa F821
         """Displays the graph of all functions loaded!
 
         :param output_file_path: the full URI of path + file name to save the dot file to.
@@ -331,21 +396,23 @@ class Driver(object):
         :param graphviz_kwargs: Optional. Kwargs to be passed to the graphviz graph object to configure it.
             E.g. dict(graph_attr={'ratio': '1'}) will set the aspect ratio to be equal of the produced image.
             See https://graphviz.org/doc/info/attrs.html for options.
+        :return: the graphviz object if you want to do more with it.
+            If returned as the result in a Jupyter Notebook cell, it will render.
         """
         try:
-            self.graph.display_all(output_file_path, render_kwargs, graphviz_kwargs)
+            return self.graph.display_all(output_file_path, render_kwargs, graphviz_kwargs)
         except ImportError as e:
             logger.warning(f"Unable to import {e}", exc_info=True)
 
     @capture_function_usage
     def visualize_execution(
         self,
-        final_vars: List[str],
+        final_vars: List[Union[str, Callable, Variable]],
         output_file_path: str,
         render_kwargs: dict,
         inputs: Dict[str, Any] = None,
         graphviz_kwargs: dict = None,
-    ):
+    ) -> Optional["graphviz.Digraph"]:  # noqa F821
         """Visualizes Execution.
 
         Note: overrides are not handled at this time.
@@ -360,11 +427,14 @@ class Driver(object):
         :param graphviz_kwargs: Optional. Kwargs to be passed to the graphviz graph object to configure it.
             E.g. dict(graph_attr={'ratio': '1'}) will set the aspect ratio to be equal of the produced image.
             See https://graphviz.org/doc/info/attrs.html for options.
+        :return: the graphviz object if you want to do more with it.
+            If returned as the result in a Jupyter Notebook cell, it will render.
         """
-        nodes, user_nodes = self.graph.get_upstream_nodes(final_vars, inputs)
-        self.validate_inputs(user_nodes, inputs)
+        _final_vars = self._create_final_vars(final_vars)
+        nodes, user_nodes = self.graph.get_upstream_nodes(_final_vars, inputs)
+        self.validate_inputs(user_nodes, inputs, nodes)
         try:
-            self.graph.display(
+            return self.graph.display(
                 nodes,
                 user_nodes,
                 output_file_path,
@@ -375,14 +445,15 @@ class Driver(object):
             logger.warning(f"Unable to import {e}", exc_info=True)
 
     @capture_function_usage
-    def has_cycles(self, final_vars: List[str]) -> bool:
+    def has_cycles(self, final_vars: List[Union[str, Callable, Variable]]) -> bool:
         """Checks that the created graph does not have cycles.
 
         :param final_vars: the outputs we want to compute.
         :return: boolean True for cycles, False for no cycles.
         """
+        _final_vars = self._create_final_vars(final_vars)
         # get graph we'd be executing over
-        nodes, user_nodes = self.graph.get_upstream_nodes(final_vars)
+        nodes, user_nodes = self.graph.get_upstream_nodes(_final_vars)
         return self.graph.has_cycles(nodes, user_nodes)
 
     @capture_function_usage
@@ -399,7 +470,7 @@ class Driver(object):
     @capture_function_usage
     def display_downstream_of(
         self, *node_names: str, output_file_path: str, render_kwargs: dict, graphviz_kwargs: dict
-    ):
+    ) -> Optional["graphviz.Digraph"]:  # noqa F821
         """Creates a visualization of the DAG starting from the passed in function name(s).
 
         Note: for any "node" visualized, we will also add its parents to the visualization as well, so
@@ -407,15 +478,17 @@ class Driver(object):
 
         :param node_names: names of function(s) that are starting points for traversing the graph.
         :param output_file_path: the full URI of path + file name to save the dot file to.
-            E.g. 'some/path/graph.dot'
+            E.g. 'some/path/graph.dot'. Pass in None to skip saving any file.
         :param render_kwargs: a dictionary of values we'll pass to graphviz render function. Defaults to viewing.
             If you do not want to view the file, pass in `{'view':False}`.
         :param graphviz_kwargs: Kwargs to be passed to the graphviz graph object to configure it.
             E.g. dict(graph_attr={'ratio': '1'}) will set the aspect ratio to be equal of the produced image.
+        :return: the graphviz object if you want to do more with it.
+            If returned as the result in a Jupyter Notebook cell, it will render.
         """
         downstream_nodes = self.graph.get_impacted_nodes(list(node_names))
         try:
-            self.graph.display(
+            return self.graph.display(
                 downstream_nodes,
                 set(),
                 output_file_path,
