@@ -2,7 +2,13 @@ import abc
 import collections
 import functools
 import logging
-from typing import Any, Callable, Collection, Dict, List, Tuple
+
+try:
+    from types import EllipsisType
+except ImportError:
+    # python3.10 and above
+    EllipsisType = type(...)
+from typing import Any, Callable, Collection, Dict, List, Optional, Union
 
 from hamilton import node, registry
 
@@ -13,7 +19,7 @@ if not registry.INITIALIZED:
     # Trigger load of extensions here because decorators are the only thing that use the registry
     # right now. Side note: ray serializes things weirdly, so we need to do this here rather than in
     # in the other choice of hamilton/base.py.
-    plugins_modules = ["pandas", "polars", "pyspark_pandas", "dask"]
+    plugins_modules = ["pandas", "polars", "pyspark_pandas", "dask", "geopandas"]
     for plugin_module in plugins_modules:
         try:
             registry.load_extension(plugin_module)
@@ -112,16 +118,45 @@ class NodeTransformLifecycle(abc.ABC):
             setattr(fn, lifecycle_name, [self])
         return fn
 
+    def required_config(self) -> Optional[List[str]]:
+        """Declares the required configuration keys for this decorator.
+        Note that these configuration keys will be filtered and passed to the `configuration`
+        parameter of the functions that this decorator uses.
+
+        Note that this currently allows for a "escape hatch".
+        That is, returning None from this function.
+
+        :return: A list of the required configuration keys.
+        """
+        return []
+
+    def optional_config(self) -> Dict[str, Any]:
+        """Declares the optional configuration keys for this decorator.
+        These are configuration keys that can be used by the decorator, but are not required.
+        Along with these we have *defaults*, which we will use to pass to the config.
+
+        :return:
+        """
+        return {}
+
+    @property
+    def name(self) -> str:
+        """Name of the decorator.
+
+        :return: The name of the decorator
+        """
+        return self.__class__.__name__
+
 
 class NodeResolver(NodeTransformLifecycle):
     """Decorator to resolve a nodes function. Can modify anything about the function and is run at DAG creation time."""
 
     @abc.abstractmethod
-    def resolve(self, fn: Callable, configuration: Dict[str, Any]) -> Callable:
+    def resolve(self, fn: Callable, config: Dict[str, Any]) -> Optional[Callable]:
         """Determines what a function resolves to. Returns None if it should not be included in the DAG.
 
         :param fn: Function to resolve
-        :param configuration: DAG config
+        :param config: DAG config
         :return: A name if it should resolve to something. Otherwise None.
         """
         pass
@@ -233,30 +268,93 @@ class NodeExpander(SubDAGModifier):
         return False
 
 
+TargetType = Union[str, Collection[str], None, EllipsisType]
+
+
 class NodeTransformer(SubDAGModifier):
-    NON_FINAL_TAG = "hamilton.decorators.non_final"  # TODO -- utilize this in _separate_final_nodes
+    def __init__(self, target: TargetType):
+        """Target determines to which node(s) this applies. This represents selection from a subDAG.
+        For the options, consider at the following graph:
+        A -> B -> C
+             \_> D -> E
+
+        1. If it is `None`, it defaults to the "old" behavior. That is, is applies to all "final" DAG
+        nodes. In the subdag. That is, all nodes with out-degree zero/sinks. In the case
+        above, *just* C and E will be transformed.
+
+        2. If it is a string, it will be interpreted as a node name. In the above case, if it is A, it
+        will transform A, B will transform B, etc...
+
+        3. If it is a collection of strings, it will be interpreted as a collection of node names.
+        That is, it will apply to all nodes that are referenced in that collection. In the above case,
+        if it is ["A", "B"], it will transform to A and B.
+
+        4. If it is Ellipsis, it will apply to all nodes in the subDAG. In the above case, it will
+        transform A, B, C, D, and E.
+
+        :param target: Which node(s)/node spec to run transforms on top of. These nodes will get
+        replaced by a list of nodes.
+        """
+        self.target = target
 
     @staticmethod
-    def _separate_final_nodes(
+    def _extract_final_nodes(
         nodes: Collection[node.Node],
-    ) -> Tuple[Collection[node.Node], Collection[node.Node]]:
+    ) -> Collection[node.Node]:
         """Separates out final nodes (sinks) from the nodes.
 
         :param nodes: Nodes to separate out
         :return: A tuple consisting of [internal, final] node sets
         """
-
-        def node_tagged_non_final(node_: node.Node):
-            return node_.tags.get(NodeTransformer.NON_FINAL_TAG, False)  # Defaults to final
-
         non_final_nodes = set()
         for node_ in nodes:
             for dep in node_.input_types:
-                if not node_tagged_non_final(node_):
-                    non_final_nodes.add(dep)
-        return [node_ for node_ in nodes if node_.name in non_final_nodes], [
-            node_ for node_ in nodes if node_.name not in non_final_nodes
-        ]
+                non_final_nodes.add(dep)
+        return [node_ for node_ in nodes if node_.name not in non_final_nodes]
+
+    @staticmethod
+    def select_nodes(target: TargetType, nodes: Collection[node.Node]) -> Collection[node.Node]:
+        """Resolves all nodes to match the target. This does a resolution on the rules
+        specified in the constructor above, giving a set of nodes that match a target.
+        We then can split them from the remainder of nodes, and just transform them.
+
+        :param target: The target to use to resolve nodes
+        :param nodes: SubDAG to resolve.
+        :return: The set of nodes matching this target
+        """
+        if target is None:
+            return NodeTransformer._extract_final_nodes(nodes)
+        elif target is Ellipsis:
+            return nodes
+        elif isinstance(target, str):
+            out = [node_ for node_ in nodes if node_.name == target]
+            if len(out) == 0:
+                raise InvalidDecoratorException(f"Could not find node {target} in {nodes}")
+            return out
+        elif isinstance(target, Collection):
+            out = [node_ for node_ in nodes if node_.name in target]
+            if len(out) != len(target):
+                raise InvalidDecoratorException(
+                    f"Could not find all nodes {target} in {nodes}. "
+                    f"Missing ({set(target) - set([node_.name for node_ in out])})"
+                )
+            return out
+        else:
+            raise ValueError(f"Invalid target: {target}")
+
+    @staticmethod
+    def compliment(
+        all_nodes: Collection[node.Node], nodes_to_transform: Collection[node.Node]
+    ) -> Collection[node.Node]:
+        """Given a set of nodes, and a set of nodes to transform, returns the set of nodes that
+        are not in the set of nodes to transform.
+
+        :param all_nodes: All nodes in the subdag
+        :param nodes_to_transform: All nodes to transform
+        :return: A collection of nodes that are not in the set of nodes to transform but are in the
+        subdag
+        """
+        return [node_ for node_ in all_nodes if node_ not in nodes_to_transform]
 
     def transform_dag(
         self, nodes: Collection[node.Node], config: Dict[str, Any], fn: Callable
@@ -269,10 +367,11 @@ class NodeTransformer(SubDAGModifier):
         :param fn: Original function that we're utilizing/modifying
         :return: The DAG of nodes in this node
         """
-        internal_nodes, final_nodes = self._separate_final_nodes(nodes)
-        out = list(internal_nodes)
-        for sink in final_nodes:
-            out += list(self.transform_node(sink, config, fn))
+        nodes_to_transform = self.select_nodes(self.target, nodes)
+        nodes_to_keep = self.compliment(nodes, nodes_to_transform)
+        out = list(nodes_to_keep)
+        for node_to_transform in nodes_to_transform:
+            out += list(self.transform_node(node_to_transform, config, fn))
         return out
 
     @abc.abstractmethod
@@ -297,6 +396,14 @@ class NodeTransformer(SubDAGModifier):
 class NodeDecorator(NodeTransformer, abc.ABC):
     DECORATE_NODES = "decorate_nodes"
 
+    def __init__(self, target: TargetType):
+        """Initializes a NodeDecorator with a target, to determine *which* nodes to decorate.
+        See documentation in NodeTransformer for more details on what to decorate.
+
+        :param target: Target parameter to resolve set of nodes to transform.
+        """
+        super().__init__(target=target)
+
     def transform_node(
         self, node_: node.Node, config: Dict[str, Any], fn: Callable
     ) -> Collection[node.Node]:
@@ -339,7 +446,7 @@ class DefaultNodeCreator(NodeCreator):
 
 
 class DefaultNodeResolver(NodeResolver):
-    def resolve(self, fn: Callable, configuration: Dict[str, Any]) -> Callable:
+    def resolve(self, fn: Callable, config: Dict[str, Any]) -> Callable:
         return fn
 
     def validate(self, fn):
@@ -357,10 +464,58 @@ class DefaultNodeExpander(NodeExpander):
 
 
 class DefaultNodeDecorator(NodeDecorator):
+    def __init__(self):
+        super().__init__(target=...)
+
     def decorate_node(self, node_: node.Node) -> node.Node:
         return node_
 
 
+def resolve_config(
+    name_for_error: str,
+    config: Dict[str, Any],
+    config_required: List[str],
+    config_optional_with_defaults: Dict[str, Any],
+) -> Dict[str, Any]:
+    """Resolves the configuration that a decorator utilizes
+
+    :param name_for_error:
+    :param config:
+    :param config_required:
+    :param config_optional_with_defaults:
+    :return:
+    """
+    missing_keys = (
+        set(config_required) - set(config.keys()) - set(config_optional_with_defaults.keys())
+    )
+    if len(missing_keys) > 0:
+        raise MissingConfigParametersException(
+            f"The following configurations are required by {name_for_error}: {missing_keys}"
+        )
+    config_out = {key: config[key] for key in config_required}
+    for key in config_optional_with_defaults:
+        config_out[key] = config.get(key, config_optional_with_defaults[key])
+    return config_out
+
+
+def filter_config(config: Dict[str, Any], decorator: NodeTransformLifecycle) -> Dict[str, Any]:
+    """Filters the config to only include the keys in config_required
+    TODO -- break this into two so we can make it easier to test.
+
+    :param config: The config to filter
+    :param config_required: The keys to include
+    :param decorator: The decorator that is utilizing the configuration
+    :return: The filtered config
+    """
+    config_required = decorator.required_config()
+    config_optional_with_defaults = decorator.optional_config()
+    if config_required is None:
+        # This is an out to allow for backwards compatibility for the config.resolve decorator
+        # Note this is an internal API, but we made the config with the `resolve` parameter public
+        return config
+    return resolve_config(decorator.name, config, config_required, config_optional_with_defaults)
+
+
 def resolve_nodes(fn: Callable, config: Dict[str, Any]) -> Collection[node.Node]:
     """Gets a list of nodes from a function. This is meant to be an abstraction between the node
     and the function that it implements. This will end up coordinating with the decorators we build
@@ -378,8 +533,7 @@ def resolve_nodes(fn: Callable, config: Dict[str, Any]) -> Collection[node.Node]
     the initial node.
 
     3. If there is a list of node expanders, apply them. Otherwise apply the default
-    nodeexpander This must be a
-    list of length one. This gives out a list of nodes.
+    node expander This must be a list of length one. This gives out a list of nodes.
 
     4. If there is a node transformer, apply that. Note that the node transformer
     gets applied individually to just the sink nodes in the subdag. It subclasses
@@ -388,26 +542,32 @@ def resolve_nodes(fn: Callable, config: Dict[str, Any]) -> Collection[node.Node]
     5. Return the final list of nodes.
 
     :param fn: Function to input.
+    :param config: Configuration to use -- this can be used by decorators to specify
+    which configuration they need.
     :return: A list of nodes into which this function transforms.
     """
     node_resolvers = getattr(fn, NodeResolver.get_lifecycle_name(), [DefaultNodeResolver()])
     for resolver in node_resolvers:
-        fn = resolver.resolve(fn, config)
+        fn = resolver.resolve(fn, config=filter_config(config, resolver))
         if fn is None:
             return []
     (node_creator,) = getattr(fn, NodeCreator.get_lifecycle_name(), [DefaultNodeCreator()])
-    nodes = node_creator.generate_nodes(fn, config)
+    nodes = node_creator.generate_nodes(fn, filter_config(config, node_creator))
     if hasattr(fn, NodeExpander.get_lifecycle_name()):
         (node_expander,) = getattr(fn, NodeExpander.get_lifecycle_name(), [DefaultNodeExpander()])
-        nodes = node_expander.transform_dag(nodes, config, fn)
+        nodes = node_expander.transform_dag(nodes, filter_config(config, node_expander), fn)
     node_transformers = getattr(fn, NodeTransformer.get_lifecycle_name(), [])
     for dag_modifier in node_transformers:
-        nodes = dag_modifier.transform_dag(nodes, config, fn)
+        nodes = dag_modifier.transform_dag(nodes, filter_config(config, dag_modifier), fn)
     node_decorators = getattr(fn, NodeDecorator.get_lifecycle_name(), [DefaultNodeDecorator()])
     for node_decorator in node_decorators:
-        nodes = node_decorator.transform_dag(nodes, config, fn)
+        nodes = node_decorator.transform_dag(nodes, filter_config(config, node_decorator), fn)
     return nodes
 
 
 class InvalidDecoratorException(Exception):
     pass
+
+
+class MissingConfigParametersException(Exception):
+    pass
