@@ -1,5 +1,7 @@
+import dataclasses
 import functools
 import inspect
+import typing
 from typing import Any, Callable, Collection, Dict, Tuple, Union
 
 import typing_inspect
@@ -14,10 +16,62 @@ from hamilton.function_modifiers.dependencies import (
     value,
 )
 
-"""Decorators that allow for les dry code by expanding one node into many"""
+"""Decorators that enables DRY code by expanding one node into many"""
 
 
 class parameterize(base.NodeExpander):
+    """Decorator to use to create many functions.
+
+    Expands a single function into n, each of which correspond to a function in which the parameter value is replaced\
+    either by:
+
+    #. A specified literal value, denoted value('literal_value').
+    #. The output from a specified upstream function (i.e. node), denoted source('upstream_function_name').
+
+    Note that ``parameterize`` can take the place of ``@parameterize_sources`` or ``@parameterize_values`` decorators \
+    below. In fact, they delegate to this!
+
+    Examples expressing different syntax:
+
+    .. code-block:: python
+
+        @parameterize(
+            # tuple of assignments (consisting of literals/upstream specifications), and docstring.
+            replace_no_parameters=({}, 'fn with no parameters replaced'),
+        )
+        def no_param_function() -> Any:
+            ...
+
+        @parameterize(
+            # tuple of assignments (consisting of literals/upstream specifications), and docstring.
+            replace_just_upstream_parameter=(
+                {'upstream_source': source('foo_source')},
+                'fn with upstream_parameter set to node foo'
+            ),
+        )
+        def param_is_upstream_function(upstream_source: Any) -> Any:
+            '''Doc string that can also be parameterized: {upstream_source}.'''
+            ...
+
+        @parameterize(
+            replace_just_literal_parameter={'literal_parameter': value('bar')},
+        )
+        def param_is_literal_value(literal_parameter: Any) -> Any:
+            '''Doc string that can also be parameterized: {literal_parameter}.'''
+            ...
+
+        @parameterize(
+            replace_both_parameters={
+                'upstream_parameter': source('foo_source'),
+                'literal_parameter': value('bar')
+            }
+        )
+        def concat(upstream_parameter: Any, literal_parameter: str) -> Any:
+            '''Adding {literal_parameter} to {upstream_parameter} to create {output_name}.'''
+            return upstream_parameter + literal_parameter
+
+    """
+
     RESERVED_KWARG = "output_name"
 
     def __init__(
@@ -26,19 +80,12 @@ class parameterize(base.NodeExpander):
             Dict[str, ParametrizedDependency], Tuple[Dict[str, ParametrizedDependency], str]
         ],
     ):
-        """Creates a parameterize decorator. For example:
-            @parameterize(
-                replace_no_parameters=({}, 'fn with no parameters replaced'),
-                replace_just_upstream_parameter=({'upstream_parameter': source('foo_source')}, 'fn with upstream_parameter set to node foo'),
-                replace_just_literal_parameter=({'literal_parameter': value('bar')}, 'fn with parameter literal_parameter set to value bar'),
-                replace_both_parameters=({'upstream_parameter': source('foo_source'), 'literal_parameter': value('bar')}, 'fn with both parameters replaced')
-            )
-            def concat(upstream_parameter: str, literal_parameter: str) -> Any:
-                return f'{upstream_parameter}{literal_parameter}'
+        """Decorator to use to create many functions.
 
-        :param parametrization: **kwargs with one of two things:
-            - a tuple of assignments (consisting of literals/upstream specifications), and docstring
-            - just assignments, in which case it parametrizes the existing docstring
+        :param parametrization: `**kwargs` with one of two things:
+
+            - a tuple of assignments (consisting of literals/upstream specifications), and docstring.
+            - just assignments, in which case it parametrizes the existing docstring.
         """
         self.parametrization = {
             key: (value[0] if isinstance(value, tuple) else value)
@@ -188,6 +235,29 @@ class parameterize(base.NodeExpander):
 
 
 class parameterize_values(parameterize):
+    """Expands a single function into n, each of which corresponds to a function in which the parameter value is \
+    replaced by that `specific value`.
+
+    .. code-block:: python
+
+        import pandas as pd
+        from hamilton.function_modifiers import parameterize_values
+        import internal_package_with_logic
+
+        ONE_OFF_DATES = {
+             #output name        # doc string               # input value to function
+            ('D_ELECTION_2016', 'US Election 2016 Dummy'): '2016-11-12',
+            ('SOME_OUTPUT_NAME', 'Doc string for this thing'): 'value to pass to function',
+        }
+                    # parameter matches the name of the argument in the function below
+        @parameterize_values(parameter='one_off_date', assigned_output=ONE_OFF_DATES)
+        def create_one_off_dates(date_index: pd.Series, one_off_date: str) -> pd.Series:
+            '''Given a date index, produces a series where a 1 is placed at the date index that would contain that event.'''
+            one_off_dates = internal_package_with_logic.get_business_week(one_off_date)
+            return internal_package_with_logic.bool_to_int(date_index.isin([one_off_dates]))
+
+    """
+
     def __init__(self, parameter: str, assigned_output: Dict[Tuple[str, str], Any]):
         """Constructor for a modifier that expands a single function into n, each of which
         corresponds to a function in which the parameter value is replaced by that *specific value*.
@@ -214,30 +284,50 @@ class parameterize_values(parameterize):
     fail_starting=(2, 0, 0),
     use_this=parameterize_values,
     explanation="We now support three parametrize decorators. @parameterize, @parameterize_values, and @parameterize_inputs",
-    migration_guide="https://github.com/stitchfix/hamilton/blob/main/decorators.md#migrating-parameterized",
+    migration_guide="https://github.com/dagworks-inc/hamilton/blob/main/decorators.md#migrating-parameterized",
 )
 class parametrized(parameterize_values):
     pass
 
 
 class parameterize_sources(parameterize):
+    """Expands a single function into `n`, each of which corresponds to a function in which the parameters specified \
+    are mapped to the specified inputs. Note this decorator and ``@parameterize_values`` are quite similar, except \
+    that the input here is another DAG node(s), i.e. column/input, rather than a specific scalar/static value.
+
+    .. code-block:: python
+
+       import pandas as pd
+       from hamilton.function_modifiers import parameterize_sources
+
+       @parameterize_sources(
+          D_ELECTION_2016_shifted=dict(one_off_date='D_ELECTION_2016'),
+          SOME_OUTPUT_NAME=dict(one_off_date='SOME_INPUT_NAME')
+       )
+       def date_shifter(one_off_date: pd.Series) -> pd.Series:
+          '''{one_off_date} shifted by 1 to create {output_name}'''
+          return one_off_date.shift(1)
+
+    """
+
     def __init__(self, **parameterization: Dict[str, Dict[str, str]]):
-        """Constructor for a modifier that expands a single function into n, each of which corresponds to replacing
+        """Constructor for a modifier that expands a single function into n, each of which corresponds to replacing\
         some subset of the specified parameters with specific upstream nodes.
 
-        Note this decorator and `@parametrized_input` are similar, except this one allows multiple
+        Note this decorator and `@parametrized_input` are similar, except this one allows multiple \
         parameters to be mapped to multiple function arguments (and it fixes the spelling mistake).
 
-        `parameterized_sources` allows you keep your code DRY by reusing the same function but replace the inputs
-        to create multiple corresponding distinct outputs. We see here that `parameterized_inputs` allows you to keep
-        your code DRY by reusing the same function to create multiple distinct outputs. The key word arguments passed
+        `parameterized_sources` allows you keep your code DRY by reusing the same function but replace the inputs \
+        to create multiple corresponding distinct outputs. We see here that `parameterized_inputs` allows you to keep \
+        your code DRY by reusing the same function to create multiple distinct outputs. The key word arguments passed \
         have to have the following structure:
             > OUTPUT_NAME = Mapping of function argument to input that should go into it.
-        The documentation for the output is taken from the function. The documentation string can be templatized with
-        the parameter names of the function and the reserved value `output_name` - those will be replaced with the
+
+        The documentation for the output is taken from the function. The documentation string can be templatized with\
+        the parameter names of the function and the reserved value `output_name` - those will be replaced with the\
         corresponding values from the parameterization.
 
-        :param **parameterization: kwargs of output name to dict of parameter mappings.
+        :param \\*\\*parameterization: kwargs of output name to dict of parameter mappings.
         """
         self.parametrization = parameterization
         if not parameterization:
@@ -262,7 +352,7 @@ class parameterize_sources(parameterize):
     fail_starting=(2, 0, 0),
     use_this=parameterize_sources,
     explanation="We now support three parametrize decorators. @parameterize, @parameterize_values, and @parameterize_inputs",
-    migration_guide="https://github.com/stitchfix/hamilton/blob/main/decorators.md#migrating-parameterized",
+    migration_guide="https://github.com/dagworks-inc/hamilton/blob/main/decorators.md#migrating-parameterized",
 )
 class parametrized_input(parameterize):
     def __init__(self, parameter: str, variable_inputs: Dict[str, Tuple[str, str]]):
@@ -275,8 +365,8 @@ class parametrized_input(parameterize):
         The `parameterized_input` allows you keep your code DRY by reusing the same function but replace the inputs
         to create multiple corresponding distinct outputs. The _parameter_ key word argument has to match one of the
         arguments in the function. The rest of the arguments are pulled from items inside the DAG.
-        The _assigned_inputs_ key word argument takes in a
-        dictionary of input_column -> tuple(Output Name, Documentation string).
+        The _assigned_inputs_ key word argument takes in a dictionary of \
+        input_column -> tuple(Output Name, Documentation string).
 
         :param parameter: Parameter to expand on.
         :param variable_inputs: A map of tuple of [parameter names, documentation] to values
@@ -300,7 +390,7 @@ class parametrized_input(parameterize):
     fail_starting=(2, 0, 0),
     use_this=parameterize_sources,
     explanation="We now support three parametrize decorators. @parameterize, @parameterize_values, and @parameterize_inputs",
-    migration_guide="https://github.com/stitchfix/hamilton/blob/main/decorators.md#migrating-parameterized",
+    migration_guide="https://github.com/dagworks-inc/hamilton/blob/main/decorators.md#migrating-parameterized",
 )
 class parameterized_inputs(parameterize_sources):
     pass
@@ -309,12 +399,14 @@ class parameterized_inputs(parameterize_sources):
 class extract_columns(base.NodeExpander):
     def __init__(self, *columns: Union[Tuple[str, str], str], fill_with: Any = None):
         """Constructor for a modifier that expands a single function into the following nodes:
+
         - n functions, each of which take in the original dataframe and output a specific column
         - 1 function that outputs the original dataframe
 
         :param columns: Columns to extract, that can be a list of tuples of (name, documentation) or just names.
-        :param fill_with: If you want to extract a column that doesn't exist, do you want to fill it with a default value?
-        Or do you want to error out? Leave empty/None to error out, set fill_value to dynamically create a column.
+        :param fill_with: If you want to extract a column that doesn't exist, do you want to fill it with a default \
+        value? Or do you want to error out? Leave empty/None to error out, set fill_value to dynamically create a \
+        column.
         """
         if not columns:
             raise base.InvalidDecoratorException(
@@ -327,13 +419,12 @@ class extract_columns(base.NodeExpander):
         self.columns = columns
         self.fill_with = fill_with
 
-    def validate(self, fn: Callable):
-        """A function is invalid if it does not output a dataframe.
-
-        :param fn: Function to validate.
-        :raises: InvalidDecoratorException If the function does not output a Dataframe
+    @staticmethod
+    def validate_return_type(fn: Callable):
+        """Validates that the return type of the function is a pandas dataframe.
+        :param fn: Function to validate
         """
-        output_type = inspect.signature(fn).return_annotation
+        output_type = typing.get_type_hints(fn).get("return")
         try:
             registry.get_column_type_from_df_type(output_type)
         except NotImplementedError:
@@ -342,6 +433,14 @@ class extract_columns(base.NodeExpander):
                 f"Error {fn} does not output a type we know about. Is it a dataframe type we support?"
             )
 
+    def validate(self, fn: Callable):
+        """A function is invalid if it does not output a dataframe.
+
+        :param fn: Function to validate.
+        :raises: InvalidDecoratorException If the function does not output a Dataframe
+        """
+        extract_columns.validate_return_type(fn)
+
     def expand_node(
         self, node_: node.Node, config: Dict[str, Any], fn: Callable
     ) -> Collection[node.Node]:
@@ -356,35 +455,60 @@ class extract_columns(base.NodeExpander):
         fn = node_.callable
         base_doc = node_.documentation
 
-        def df_generator(*args, **kwargs) -> Any:
-            df_generated = fn(*args, **kwargs)
-            if self.fill_with is not None:
-                for col in self.columns:
-                    if col not in df_generated:
-                        registry.fill_with_scalar(df_generated, col, self.fill_with)
-                        assert col in df_generated
-            return df_generated
+        # if fn is an async function
+        if inspect.iscoroutinefunction(fn):
+
+            async def df_generator(*args, **kwargs) -> Any:
+                df_generated = await fn(*args, **kwargs)
+                if self.fill_with is not None:
+                    for col in self.columns:
+                        if col not in df_generated:
+                            registry.fill_with_scalar(df_generated, col, self.fill_with)
+                            assert col in df_generated
+                return df_generated
+
+        else:
+
+            def df_generator(*args, **kwargs) -> Any:
+                df_generated = fn(*args, **kwargs)
+                if self.fill_with is not None:
+                    for col in self.columns:
+                        if col not in df_generated:
+                            registry.fill_with_scalar(df_generated, col, self.fill_with)
+                            assert col in df_generated
+                return df_generated
 
         output_nodes = [node_.copy_with(callabl=df_generator)]
-
         output_type = node_.type
         series_type = registry.get_column_type_from_df_type(output_type)
-
         for column in self.columns:
             doc_string = base_doc  # default doc string of base function.
             if isinstance(column, Tuple):  # Expand tuple into constituents
                 column, doc_string = column
 
-            def extractor_fn(
-                column_to_extract: str = column, **kwargs
-            ) -> Any:  # avoiding problems with closures
-                df = kwargs[node_.name]
-                if column_to_extract not in df:
-                    raise base.InvalidDecoratorException(
-                        f"No such column: {column_to_extract} produced by {node_.name}. "
-                        f"It only produced {str(df.columns)}"
-                    )
-                return registry.get_column(df, column_to_extract)
+            if inspect.iscoroutinefunction(fn):
+
+                async def extractor_fn(column_to_extract: str = column, **kwargs) -> Any:
+                    df = kwargs[node_.name]
+                    if column_to_extract not in df:
+                        raise base.InvalidDecoratorException(
+                            f"No such column: {column_to_extract} produced by {node_.name}. "
+                            f"It only produced {str(df.columns)}"
+                        )
+                    return registry.get_column(df, column_to_extract)
+
+            else:
+
+                def extractor_fn(
+                    column_to_extract: str = column, **kwargs
+                ) -> Any:  # avoiding problems with closures
+                    df = kwargs[node_.name]
+                    if column_to_extract not in df:
+                        raise base.InvalidDecoratorException(
+                            f"No such column: {column_to_extract} produced by {node_.name}. "
+                            f"It only produced {str(df.columns)}"
+                        )
+                    return registry.get_column(df, column_to_extract)
 
             output_nodes.append(
                 node.Node(
@@ -404,12 +528,14 @@ class extract_fields(base.NodeExpander):
 
     def __init__(self, fields: dict, fill_with: Any = None):
         """Constructor for a modifier that expands a single function into the following nodes:
+
         - n functions, each of which take in the original dict and output a specific field
         - 1 function that outputs the original dict
 
         :param fields: Fields to extract. A dict of 'field_name' -> 'field_type'.
-        :param fill_with: If you want to extract a field that doesn't exist, do you want to fill it with a default value?
-        Or do you want to error out? Leave empty/None to error out, set fill_value to dynamically create a field value.
+        :param fill_with: If you want to extract a field that doesn't exist, do you want to fill it with a default \
+        value? Or do you want to error out? Leave empty/None to error out, set fill_value to dynamically create a \
+        field value.
         """
         if not fields:
             raise base.InvalidDecoratorException(
@@ -442,7 +568,7 @@ class extract_fields(base.NodeExpander):
         :param fn: Function to validate.
         :raises: InvalidDecoratorException If the function is not annotated with a dict or typing.Dict type as output.
         """
-        output_type = inspect.signature(fn).return_annotation
+        output_type = typing.get_type_hints(fn).get("return")
         if typing_inspect.is_generic_type(output_type):
             base_type = typing_inspect.get_origin(output_type)
             if (
@@ -475,29 +601,56 @@ class extract_fields(base.NodeExpander):
         fn = node_.callable
         base_doc = node_.documentation
 
-        def dict_generator(*args, **kwargs):
-            dict_generated = fn(*args, **kwargs)
-            if self.fill_with is not None:
-                for field in self.fields:
-                    if field not in dict_generated:
-                        dict_generated[field] = self.fill_with
-            return dict_generated
+        # if fn is async
+        if inspect.iscoroutinefunction(fn):
+
+            async def dict_generator(*args, **kwargs):
+                dict_generated = await fn(*args, **kwargs)
+                if self.fill_with is not None:
+                    for field in self.fields:
+                        if field not in dict_generated:
+                            dict_generated[field] = self.fill_with
+                return dict_generated
+
+        else:
+
+            def dict_generator(*args, **kwargs):
+                dict_generated = fn(*args, **kwargs)
+                if self.fill_with is not None:
+                    for field in self.fields:
+                        if field not in dict_generated:
+                            dict_generated[field] = self.fill_with
+                return dict_generated
 
         output_nodes = [node_.copy_with(callabl=dict_generator)]
 
         for field, field_type in self.fields.items():
             doc_string = base_doc  # default doc string of base function.
 
-            def extractor_fn(
-                field_to_extract: str = field, **kwargs
-            ) -> field_type:  # avoiding problems with closures
-                dt = kwargs[node_.name]
-                if field_to_extract not in dt:
-                    raise base.InvalidDecoratorException(
-                        f"No such field: {field_to_extract} produced by {node_.name}. "
-                        f"It only produced {list(dt.keys())}"
-                    )
-                return kwargs[node_.name][field_to_extract]
+            # if fn is async
+            if inspect.iscoroutinefunction(fn):
+
+                async def extractor_fn(field_to_extract: str = field, **kwargs) -> field_type:
+                    dt = kwargs[node_.name]
+                    if field_to_extract not in dt:
+                        raise base.InvalidDecoratorException(
+                            f"No such field: {field_to_extract} produced by {node_.name}. "
+                            f"It only produced {list(dt.keys())}"
+                        )
+                    return kwargs[node_.name][field_to_extract]
+
+            else:
+
+                def extractor_fn(
+                    field_to_extract: str = field, **kwargs
+                ) -> field_type:  # avoiding problems with closures
+                    dt = kwargs[node_.name]
+                    if field_to_extract not in dt:
+                        raise base.InvalidDecoratorException(
+                            f"No such field: {field_to_extract} produced by {node_.name}. "
+                            f"It only produced {list(dt.keys())}"
+                        )
+                    return kwargs[node_.name][field_to_extract]
 
             output_nodes.append(
                 node.Node(
@@ -510,3 +663,104 @@ class extract_fields(base.NodeExpander):
                 )
             )
         return output_nodes
+
+
+@dataclasses.dataclass
+class ParameterizedExtract:
+    """Dataclass to hold inputs for @parameterize and @parameterize_extract_columns.
+
+    :param outputs: A tuple of strings, each of which is the name of an output.
+    :param input_mapping: A dictionary of string to ParametrizedDependency. The string is the name of the python \
+    parameter of the decorated function, and the value is a "source"/"value" which will be passed as input for that\
+    parameter to the function.
+    """
+
+    outputs: Tuple[str, ...]
+    input_mapping: Dict[str, ParametrizedDependency]
+
+
+class parameterize_extract_columns(base.NodeExpander):
+    """`@parameterize_extract_columns` gives you the power of both `@extract_columns` and `@parameterize` in one\
+     decorator.
+
+    It takes in a list of `Parameterized_Extract` objects, each of which is composed of:
+    1. A list of columns to extract, and
+    2. A parameterization that gets used
+
+    In the following case, we produce four columns, two for each parameterization:
+
+    .. code-block:: python
+
+        import pandas as pd
+        from function_modifiers import parameterize_extract_columns, ParameterizedExtract, source, value
+        @parameterize_extract_columns(
+            ParameterizedExtract(
+                ("outseries1a", "outseries2a"),
+                {"input1": source("inseries1a"), "input2": source("inseries1b"), "input3": value(10)},
+            ),
+            ParameterizedExtract(
+                ("outseries1b", "outseries2b"),
+                {"input1": source("inseries2a"), "input2": source("inseries2b"), "input3": value(100)},
+            ),
+        )
+        def fn(input1: pd.Series, input2: pd.Series, input3: float) -> pd.DataFrame:
+            return pd.concat([input1 * input2 * input3, input1 + input2 + input3], axis=1)
+
+    """
+
+    def __init__(self, *extract_config: ParameterizedExtract, reassign_columns: bool = True):
+        """Initializes a `parameterized_extract` decorator. Note this currently works for series,
+        but the plan is to extend it to fields as well...
+
+        :param extract_config: A configuration consisting of a list ParameterizedExtract classes\
+        These contain the information of a `@parameterized` and `@extract...` together.
+        :param reassign_columns: Whether we want to reassign the columns as part of the function.
+        """
+        self.extract_config = extract_config
+        self.reassign_columns = reassign_columns
+
+    def expand_node(
+        self, node_: node.Node, config: Dict[str, Any], fn: Callable
+    ) -> Collection[node.Node]:
+        """Expands a node into multiple, given the extract_config passed to
+        parameterize_extract_columns. Goes through all parameterizations,
+        creates an extract_columns node for each, then delegates to that.
+        Note this calls out to `@parameterize` and `@extract_columns` rather
+        than reimplementing the logic.
+
+        :param node_: Node to expand
+        :param config: Config to use to expand
+        :param fn: Original function
+        :return: The nodes produced by this decorator.
+        """
+        output_nodes = []
+        for i, parameterization in enumerate(self.extract_config):
+
+            @functools.wraps(fn)
+            def wrapper_fn(*args, _output_columns=parameterization.outputs, **kwargs):
+                df_out = fn(*args, **kwargs)
+                df_out.columns = _output_columns
+                return df_out
+
+            new_node = node_.copy_with(callabl=wrapper_fn)
+            fn_to_call = wrapper_fn if self.reassign_columns else fn
+            # We have to rename the underlying function so that we do not
+            # get naming collisions. Using __ is cleaner than using a uuid
+            # as it is easier to read/manage and naturally maeks sense.
+            parameterization_decorator = parameterize(
+                **{node_.name + f"__{i}": parameterization.input_mapping}
+            )
+            (parameterized_node,) = parameterization_decorator.expand_node(
+                new_node, config, fn_to_call
+            )
+            extract_columns_decorator = extract_columns(*parameterization.outputs)
+            output_nodes.extend(
+                extract_columns_decorator.expand_node(
+                    parameterized_node, config, parameterized_node.callable
+                )
+            )
+
+        return output_nodes
+
+    def validate(self, fn: Callable):
+        extract_columns.validate_return_type(fn)
